@@ -12,21 +12,38 @@ typedef struct fz_xml_s fz_xml;
 
 /*
 	fz_parse_xml: Parse a zero-terminated string into a tree of xml nodes.
+
+	preserve_white: whether to keep or delete all-whitespace nodes.
 */
-fz_xml *fz_parse_xml(fz_context *ctx, unsigned char *buf, int len);
+fz_xml *fz_parse_xml(fz_context *ctx, unsigned char *buf, int len, int preserve_white);
+
+/*
+	fz_xml_prev: Return previous sibling of XML node.
+*/
+fz_xml *fz_xml_prev(fz_xml *item);
 
 /*
 	fz_xml_next: Return next sibling of XML node.
 */
 fz_xml *fz_xml_next(fz_xml *item);
 
+/*
+	fz_xml_up: Return parent of XML node.
+*/
+fz_xml *fz_xml_up(fz_xml *item);
+
 /*
 	fz_xml_down: Return first child of XML node.
 */
 fz_xml *fz_xml_down(fz_xml *item);
 
 /*
-	fz_xml_tag: Return tag of XML node. Return the empty string for text nodes.
+	fz_xml_is_tag: Return true if the tag name matches.
+*/
+int fz_xml_is_tag(fz_xml *item, const char *name);
+
+/*
+	fz_xml_tag: Return tag of XML node. Return NULL for text nodes.
 */
 char *fz_xml_tag(fz_xml *item);
 
@@ -38,7 +55,7 @@ char *fz_xml_att(fz_xml *item, const char *att);
 
 /*
 	fz_xml_text: Return the text content of an XML node.
-	NULL if the node is a tag.
+	Return NULL if the node is a tag.
 */
 char *fz_xml_text(fz_xml *item);
 
