@@ -23,6 +23,25 @@ unsigned char *pdf_crypt_key(pdf_document *doc);
 void pdf_print_crypt(pdf_crypt *crypt);
 #endif
 
+typedef struct pdf_designated_name_s
+{
+	char *cn;
+	char *o;
+	char *ou;
+	char *email;
+	char *c;
+}
+pdf_designated_name;
+
+void pdf_free_designated_name(pdf_designated_name *dn);
+
+
+pdf_signer *pdf_read_pfx(fz_context *ctx, const char *sigfile, const char *password);
+pdf_signer *pdf_keep_signer(pdf_signer *signer);
+void pdf_drop_signer(pdf_signer *signer);
+pdf_designated_name *pdf_signer_designated_name(pdf_signer *signer);
+void pdf_write_digest(pdf_document *doc, char *filename, pdf_obj *byte_range, int digest_offset, int digest_length, pdf_signer *signer);
+
 /*
 	pdf_signature_widget_byte_range: retrieve the byte range for a signature widget
 */
@@ -34,8 +53,13 @@ int pdf_signature_widget_byte_range(pdf_document *doc, pdf_widget *widget, int (
 int pdf_signature_widget_contents(pdf_document *doc, pdf_widget *widget, char **contents);
 
 /*
-	fz_check_signature: check a signature's certificate chain and digest
+	pdf_check_signature: check a signature's certificate chain and digest
 */
-int pdf_check_signature(fz_context *ctx, pdf_document *doc, pdf_widget *widget, char *file, char *ebuf, int ebufsize);
+int pdf_check_signature(pdf_document *doc, pdf_widget *widget, char *file, char *ebuf, int ebufsize);
+
+/*
+	pdf_sign_signature: sign a signature form field
+*/
+void pdf_sign_signature(pdf_document *doc, pdf_widget *widget, const char *sigfile, const char *password);
 
 #endif
