@@ -9,6 +9,7 @@ import android.app.Activity;
 import android.app.AlertDialog;
 import android.content.Context;
 import android.content.DialogInterface;
+import android.content.DialogInterface.OnCancelListener;
 import android.content.Intent;
 import android.content.SharedPreferences;
 import android.content.res.Resources;
@@ -222,12 +223,12 @@ public class MuPDFActivity extends Activity implements FilePicker.FilePickerSupp
 		return core;
 	}
 
-	private MuPDFCore openBuffer(byte buffer[])
+	private MuPDFCore openBuffer(byte buffer[], String magic)
 	{
 		System.out.println("Trying to open byte buffer");
 		try
 		{
-			core = new MuPDFCore(this, buffer);
+			core = new MuPDFCore(this, buffer, magic);
 			// New file: drop the old outline data
 			OutlineActivityData.set(null);
 		}
@@ -259,52 +260,62 @@ public class MuPDFActivity extends Activity implements FilePicker.FilePickerSupp
 			byte buffer[] = null;
 			if (Intent.ACTION_VIEW.equals(intent.getAction())) {
 				Uri uri = intent.getData();
+				System.out.println("URI to open is: " + uri);
 				if (uri.toString().startsWith("content://")) {
-					// Handle view requests from the Transformer Prime's file manager
-					// Hopefully other file managers will use this same scheme, if not
-					// using explicit paths.
-					Cursor cursor = getContentResolver().query(uri, new String[]{"_data"}, null, null, null);
-					if (cursor.moveToFirst()) {
-						String str = cursor.getString(0);
-						String reason = null;
-						if (str == null) {
-							try {
-								InputStream is = getContentResolver().openInputStream(uri);
-								int len = is.available();
-								buffer = new byte[len];
-								is.read(buffer, 0, len);
-								is.close();
+					String reason = null;
+					try {
+						InputStream is = getContentResolver().openInputStream(uri);
+						int len = is.available();
+						buffer = new byte[len];
+						is.read(buffer, 0, len);
+						is.close();
+					}
+					catch (java.lang.OutOfMemoryError e) {
+						System.out.println("Out of memory during buffer reading");
+						reason = e.toString();
+					}
+					catch (Exception e) {
+						System.out.println("Exception reading from stream: " + e);
+
+						// Handle view requests from the Transformer Prime's file manager
+						// Hopefully other file managers will use this same scheme, if not
+						// using explicit paths.
+						// I'm hoping that this case below is no longer needed...but it's
+						// hard to test as the file manager seems to have changed in 4.x.
+						try {
+							Cursor cursor = getContentResolver().query(uri, new String[]{"_data"}, null, null, null);
+							if (cursor.moveToFirst()) {
+								String str = cursor.getString(0);
+								if (str == null) {
+									reason = "Couldn't parse data in intent";
+								}
+								else {
+									uri = Uri.parse(str);
+								}
 							}
-							catch (java.lang.OutOfMemoryError e)
-							{
-								System.out.println("Out of memory during buffer reading");
-								reason = e.toString();
-							}
-							catch (Exception e) {
-								reason = e.toString();
-							}
-							if (reason != null)
-							{
-								buffer = null;
-								Resources res = getResources();
-								AlertDialog alert = mAlertBuilder.create();
-								setTitle(String.format(res.getString(R.string.cannot_open_document_Reason), reason));
-								alert.setButton(AlertDialog.BUTTON_POSITIVE, getString(R.string.dismiss),
-										new DialogInterface.OnClickListener() {
-											public void onClick(DialogInterface dialog, int which) {
-												finish();
-											}
-										});
-								alert.show();
-								return;
-							}
-						} else {
-							uri = Uri.parse(str);
 						}
+						catch (Exception e2) {
+							System.out.println("Exception in Transformer Prime file manager code: " + e2);
+							reason = e2.toString();
+						}
+					}
+					if (reason != null) {
+						buffer = null;
+						Resources res = getResources();
+						AlertDialog alert = mAlertBuilder.create();
+						setTitle(String.format(res.getString(R.string.cannot_open_document_Reason), reason));
+						alert.setButton(AlertDialog.BUTTON_POSITIVE, getString(R.string.dismiss),
+								new DialogInterface.OnClickListener() {
+									public void onClick(DialogInterface dialog, int which) {
+										finish();
+									}
+								});
+						alert.show();
+						return;
 					}
 				}
 				if (buffer != null) {
-					core = openBuffer(buffer);
+					core = openBuffer(buffer, intent.getType());
 				} else {
 					core = openFile(Uri.decode(uri.getEncodedPath()));
 				}
@@ -329,6 +340,13 @@ public class MuPDFActivity extends Activity implements FilePicker.FilePickerSupp
 							finish();
 						}
 					});
+			alert.setOnCancelListener(new OnCancelListener() {
+
+				@Override
+				public void onCancel(DialogInterface dialog) {
+					finish();
+				}
+			});
 			alert.show();
 			return;
 		}
@@ -475,7 +493,7 @@ public class MuPDFActivity extends Activity implements FilePicker.FilePickerSupp
 			}
 		});
 
-		if (core.fileFormat().startsWith("PDF"))
+		if (core.fileFormat().startsWith("PDF") && core.isUnencryptedPDF() && !core.wasOpenedFromBuffer())
 		{
 			mAnnotButton.setOnClickListener(new View.OnClickListener() {
 				public void onClick(View v) {
@@ -1083,7 +1101,7 @@ public class MuPDFActivity extends Activity implements FilePicker.FilePickerSupp
 
 	@Override
 	public void onBackPressed() {
-		if (core.hasChanges()) {
+		if (core != null && core.hasChanges()) {
 			DialogInterface.OnClickListener listener = new DialogInterface.OnClickListener() {
 				public void onClick(DialogInterface dialog, int which) {
 					if (which == AlertDialog.BUTTON_POSITIVE)
