@@ -2,6 +2,12 @@
 
 #import "MuLibraryController.h"
 
+enum
+{
+	// use at most 128M for resource cache
+	ResourceCacheMaxSize = 128<<20	// use at most 128M for resource cache
+};
+
 @interface MuAppDelegate : NSObject <UIApplicationDelegate, UINavigationControllerDelegate>
 {
 	UIWindow *window;
