@@ -10,16 +10,30 @@
 	if (self)
 	{
 		dispatch_sync(queue, ^{});
-		doc = fz_open_document(ctx, aFilename);
-		if (!doc)
+
+		fz_var(self);
+
+		fz_try(ctx)
 		{
-			self = nil;
+			doc = fz_open_document(ctx, aFilename);
+			if (!doc)
+			{
+				[self release];
+				self = nil;
+			}
+			else
+			{
+				pdf_document *idoc = pdf_specifics(doc);
+				if (idoc) pdf_enable_js(idoc);
+				interactive = (idoc != NULL) && (pdf_crypt_version(idoc) == 0);
+			}
 		}
-		else
+		fz_catch(ctx)
 		{
-			pdf_document *idoc = pdf_specifics(doc);
-			if (idoc) pdf_enable_js(idoc);
-			interactive = (idoc != NULL);
+			if (doc != NULL)
+				fz_close_document(doc);
+			[self release];
+			self = nil;
 		}
 	}
 	return self;
