@@ -14,7 +14,7 @@
 #define DEBUG_MESSAGE(A) do { } while(0)
 #endif
 
-#define BLOCK_SHIFT 12
+#define BLOCK_SHIFT 20
 #define BLOCK_SIZE (1<<BLOCK_SHIFT)
 
 #ifdef _WIN32
@@ -45,6 +45,7 @@ struct curl_stream_state_s
 	int kill_thread;
 	void (*more_data)(void *, int);
 	void *more_data_arg;
+	const char *error;
 
 	unsigned char public_buffer[4096];
 
@@ -224,7 +225,7 @@ static size_t data_arrived(void *ptr, size_t size, size_t nmemb, void *state_)
 		}
 	}
 
-	DEBUG_MESSAGE((state->ctx, "data arrived: offset=%d len=%d", state->current_fill_point, size));
+	DEBUG_MESSAGE((state->ctx, "data arrived: offset=%d len=%d", state->current_fill_point, (int) size));
 	old_start = state->current_fill_point;
 	memcpy(state->buffer + state->current_fill_point, ptr, size);
 	state->current_fill_point += size;
@@ -252,6 +253,7 @@ fetch_chunk(curl_stream_state *state)
 {
 	char text[32];
 	int fill, start, end;
+	CURLcode ret;
 
 	lock(state);
 
@@ -302,7 +304,9 @@ fetch_chunk(curl_stream_state *state)
 		end = state->content_length-1;
 	snprintf(text, 32, "%d-%d", start, end);
 	curl_easy_setopt(state->handle, CURLOPT_RANGE, text);
-	curl_easy_perform(state->handle);
+	ret = curl_easy_perform(state->handle);
+	if (ret != CURLE_OK)
+		state->error = curl_easy_strerror(ret);
 }
 
 static void
@@ -322,6 +326,9 @@ stream_next(fz_stream *stream, int len)
 	int left_over = (-read_point) & (BLOCK_SIZE-1);
 	unsigned char *buf = state->public_buffer;
 
+	if (state->error != NULL)
+		fz_throw(stream->ctx, FZ_ERROR_GENERIC, "cannot fetch data: %s", state->error);
+
 	if (len > sizeof(state->public_buffer))
 		len = sizeof(state->public_buffer);
 
