@@ -142,6 +142,27 @@ fz_clear_pixmap(fz_context *ctx, fz_pixmap *pix)
 void
 fz_clear_pixmap_with_value(fz_context *ctx, fz_pixmap *pix, int value)
 {
+	/* CMYK needs special handling (and potentially any other subtractive colorspaces) */
+	if (pix->colorspace && pix->colorspace->n == 4)
+	{
+		int x, y;
+		unsigned char *s = pix->samples;
+
+		value = 255 - value;
+		for (y = 0; y < pix->h; y++)
+		{
+			for (x = 0; x < pix->w; x++)
+			{
+				*s++ = 0;
+				*s++ = 0;
+				*s++ = 0;
+				*s++ = value;
+				*s++ = 255;
+			}
+		}
+		return;
+	}
+
 	if (value == 255)
 	{
 		memset(pix->samples, 255, (unsigned int)(pix->w * pix->h * pix->n));
@@ -280,14 +301,39 @@ fz_clear_pixmap_rect_with_value(fz_context *ctx, fz_pixmap *dest, int value, con
 
 	destspan = dest->w * dest->n;
 	destp = dest->samples + (unsigned int)(destspan * (local_b.y0 - dest->y) + dest->n * (local_b.x0 - dest->x));
+
+	/* CMYK needs special handling (and potentially any other subtractive colorspaces) */
+	if (dest->colorspace && dest->colorspace->n == 4)
+	{
+		value = 255 - value;
+		do
+		{
+			unsigned char *s = destp;
+			for (x = 0; x < w; x++)
+			{
+				*s++ = 0;
+				*s++ = 0;
+				*s++ = 0;
+				*s++ = value;
+				*s++ = 255;
+			}
+			destp += destspan;
+		}
+		while (--y);
+		return;
+	}
+
 	if (value == 255)
+	{
 		do
 		{
 			memset(destp, 255, (unsigned int)(w * dest->n));
 			destp += destspan;
 		}
 		while (--y);
+	}
 	else
+	{
 		do
 		{
 			unsigned char *s = destp;
@@ -300,6 +346,7 @@ fz_clear_pixmap_rect_with_value(fz_context *ctx, fz_pixmap *dest, int value, con
 			destp += destspan;
 		}
 		while (--y);
+	}
 }
 
 void
@@ -367,6 +414,53 @@ fz_alpha_from_gray(fz_context *ctx, fz_pixmap *gray, int luminosity)
 	return alpha;
 }
 
+void
+fz_tint_pixmap(fz_context *ctx, fz_pixmap *pix, int r, int g, int b)
+{
+	unsigned char *s = pix->samples;
+	int x, y;
+
+	if (pix->colorspace == fz_device_bgr(ctx))
+	{
+		int save = r;
+		r = b;
+		b = save;
+	}
+	else if (pix->colorspace == fz_device_gray(ctx))
+	{
+		g = (r + g + b) / 3;
+	}
+	else if (pix->colorspace != fz_device_rgb(ctx))
+	{
+		fz_throw(ctx, FZ_ERROR_GENERIC, "can only tint RGB, BGR and Gray pixmaps");
+	}
+
+	if (pix->n == 4)
+	{
+		for (x = 0; x < pix->w; x++)
+		{
+			for (y = 0; y < pix->h; y++)
+			{
+				s[0] = fz_mul255(s[0], r);
+				s[1] = fz_mul255(s[1], g);
+				s[2] = fz_mul255(s[2], b);
+				s += 4;
+			}
+		}
+	}
+	else if (pix->n == 2)
+	{
+		for (x = 0; x < pix->w; x++)
+		{
+			for (y = 0; y < pix->h; y++)
+			{
+				*s = fz_mul255(*s, g);
+				s += 2;
+			}
+		}
+	}
+}
+
 void
 fz_invert_pixmap(fz_context *ctx, fz_pixmap *pix)
 {
@@ -432,52 +526,64 @@ fz_gamma_pixmap(fz_context *ctx, fz_pixmap *pix, float gamma)
  */
 
 void
-fz_write_pnm(fz_context *ctx, fz_pixmap *pixmap, char *filename)
+fz_output_pnm_header(fz_output *out, int w, int h, int n)
 {
-	FILE *fp;
-	unsigned char *p;
-	int len;
+	fz_context *ctx = out->ctx;
 
-	if (pixmap->n != 1 && pixmap->n != 2 && pixmap->n != 4)
+	if (n != 1 && n != 2 && n != 4)
 		fz_throw(ctx, FZ_ERROR_GENERIC, "pixmap must be grayscale or rgb to write as pnm");
 
-	fp = fopen(filename, "wb");
-	if (!fp)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot open file '%s': %s", filename, strerror(errno));
+	if (n == 1 || n == 2)
+		fz_printf(out, "P5\n");
+	if (n == 4)
+		fz_printf(out, "P6\n");
+	fz_printf(out, "%d %d\n", w, h);
+	fz_printf(out, "255\n");
+}
 
-	if (pixmap->n == 1 || pixmap->n == 2)
-		fprintf(fp, "P5\n");
-	if (pixmap->n == 4)
-		fprintf(fp, "P6\n");
-	fprintf(fp, "%d %d\n", pixmap->w, pixmap->h);
-	fprintf(fp, "255\n");
+void
+fz_output_pnm_band(fz_output *out, int w, int h, int n, int band, int bandheight, unsigned char *p)
+{
+	int len;
+	int start = band * bandheight;
+	int end = start + bandheight;
 
-	len = pixmap->w * pixmap->h;
-	p = pixmap->samples;
+	if (end > h)
+		end = h;
+	end -= start;
 
-	switch (pixmap->n)
+	len = w * end;
+
+	switch (n)
 	{
 	case 1:
-		fwrite(p, 1, len, fp);
+		fz_write(out, p, len);
 		break;
 	case 2:
 		while (len--)
 		{
-			putc(p[0], fp);
+			fz_putc(out, p[0]);
 			p += 2;
 		}
 		break;
 	case 4:
 		while (len--)
 		{
-			putc(p[0], fp);
-			putc(p[1], fp);
-			putc(p[2], fp);
+			fz_putc(out, p[0]);
+			fz_putc(out, p[1]);
+			fz_putc(out, p[2]);
 			p += 4;
 		}
 	}
+}
 
-	fclose(fp);
+void
+fz_write_pnm(fz_context *ctx, fz_pixmap *pixmap, char *filename)
+{
+	fz_output *out = fz_new_output_to_filename(ctx, filename);
+	fz_output_pnm_header(out, pixmap->w, pixmap->h, pixmap->n);
+	fz_output_pnm_band(out, pixmap->w, pixmap->h, pixmap->n, 0, pixmap->h, pixmap->samples);
+	fz_close_output(out);
 }
 
 /*
@@ -485,50 +591,61 @@ fz_write_pnm(fz_context *ctx, fz_pixmap *pixmap, char *filename)
  */
 
 void
-fz_write_pam(fz_context *ctx, fz_pixmap *pixmap, char *filename, int savealpha)
+fz_output_pam_header(fz_output *out, int w, int h, int n, int savealpha)
 {
-	unsigned char *sp;
-	int y, w, k;
-	FILE *fp;
-
-	int sn = pixmap->n;
-	int dn = pixmap->n;
+	int sn = n;
+	int dn = n;
 	if (!savealpha && dn > 1)
 		dn--;
 
-	fp = fopen(filename, "wb");
-	if (!fp)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot open file '%s': %s", filename, strerror(errno));
+	fz_printf(out, "P7\n");
+	fz_printf(out, "WIDTH %d\n", w);
+	fz_printf(out, "HEIGHT %d\n", h);
+	fz_printf(out, "DEPTH %d\n", dn);
+	fz_printf(out, "MAXVAL 255\n");
+	if (dn == 1) fz_printf(out, "TUPLTYPE GRAYSCALE\n");
+	else if (dn == 2 && sn == 2) fz_printf(out, "TUPLTYPE GRAYSCALE_ALPHA\n");
+	else if (dn == 3 && sn == 4) fz_printf(out, "TUPLTYPE RGB\n");
+	else if (dn == 4 && sn == 4) fz_printf(out, "TUPLTYPE RGB_ALPHA\n");
+	else if (dn == 4 && sn == 5) fz_printf(out, "TUPLTYPE CMYK\n");
+	else if (dn == 5 && sn == 5) fz_printf(out, "TUPLTYPE CMYK_ALPHA\n");
+	fz_printf(out, "ENDHDR\n");
+}
 
-	fprintf(fp, "P7\n");
-	fprintf(fp, "WIDTH %d\n", pixmap->w);
-	fprintf(fp, "HEIGHT %d\n", pixmap->h);
-	fprintf(fp, "DEPTH %d\n", dn);
-	fprintf(fp, "MAXVAL 255\n");
-	if (pixmap->colorspace)
-		fprintf(fp, "# COLORSPACE %s\n", pixmap->colorspace->name);
-	switch (dn)
-	{
-	case 1: fprintf(fp, "TUPLTYPE GRAYSCALE\n"); break;
-	case 2: if (sn == 2) fprintf(fp, "TUPLTYPE GRAYSCALE_ALPHA\n"); break;
-	case 3: if (sn == 4) fprintf(fp, "TUPLTYPE RGB\n"); break;
-	case 4: if (sn == 4) fprintf(fp, "TUPLTYPE RGB_ALPHA\n"); break;
-	}
-	fprintf(fp, "ENDHDR\n");
+void
+fz_output_pam_band(fz_output *out, int w, int h, int n, int band, int bandheight, unsigned char *sp, int savealpha)
+{
+	int y, x, k;
+	int start = band * bandheight;
+	int end = start + bandheight;
+	int sn = n;
+	int dn = n;
+	if (!savealpha && dn > 1)
+		dn--;
 
-	sp = pixmap->samples;
-	for (y = 0; y < pixmap->h; y++)
+	if (end > h)
+		end = h;
+	end -= start;
+
+	for (y = 0; y < end; y++)
 	{
-		w = pixmap->w;
-		while (w--)
+		x = w;
+		while (x--)
 		{
 			for (k = 0; k < dn; k++)
-				putc(sp[k], fp);
+				fz_putc(out, sp[k]);
 			sp += sn;
 		}
 	}
+}
 
-	fclose(fp);
+void
+fz_write_pam(fz_context *ctx, fz_pixmap *pixmap, char *filename, int savealpha)
+{
+	fz_output *out = fz_new_output_to_filename(ctx, filename);
+	fz_output_pam_header(out, pixmap->w, pixmap->h, pixmap->n, savealpha);
+	fz_output_pam_band(out, pixmap->w, pixmap->h, pixmap->n, 0, pixmap->h, pixmap->samples, savealpha);
+	fz_close_output(out);
 }
 
 /*
@@ -560,25 +677,20 @@ static void putchunk(char *tag, unsigned char *data, int size, fz_output *out)
 void
 fz_write_png(fz_context *ctx, fz_pixmap *pixmap, char *filename, int savealpha)
 {
-	FILE *fp = fopen(filename, "wb");
-	fz_output *out = NULL;
+	fz_output *out = fz_new_output_to_filename(ctx, filename);
+	fz_png_output_context *poc = NULL;
 
-	if (!fp)
-	{
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot open file '%s': %s", filename, strerror(errno));
-	}
-
-	fz_var(out);
+	fz_var(poc);
 
 	fz_try(ctx)
 	{
-		out = fz_new_output_with_file(ctx, fp);
-		fz_output_png(out, pixmap, savealpha);
+		poc = fz_output_png_header(out, pixmap->w, pixmap->h, pixmap->n, savealpha);
+		fz_output_png_band(out, pixmap->w, pixmap->h, pixmap->n, 0, pixmap->h, pixmap->samples, savealpha, poc);
 	}
 	fz_always(ctx)
 	{
+		fz_output_png_trailer(out, poc);
 		fz_close_output(out);
-		fclose(fp);
 	}
 	fz_catch(ctx)
 	{
@@ -589,34 +701,60 @@ fz_write_png(fz_context *ctx, fz_pixmap *pixmap, char *filename, int savealpha)
 void
 fz_output_png(fz_output *out, const fz_pixmap *pixmap, int savealpha)
 {
-	static const unsigned char pngsig[8] = { 137, 80, 78, 71, 13, 10, 26, 10 };
-	unsigned char head[13];
-	unsigned char *udata = NULL;
-	unsigned char *cdata = NULL;
-	unsigned char *sp, *dp;
-	uLong usize, csize;
-	int y, x, k, sn, dn;
-	int color;
-	int err;
+	fz_png_output_context *poc;
 	fz_context *ctx;
 
-	if (!out || !pixmap)
+	if (!out)
 		return;
 
 	ctx = out->ctx;
+	poc = fz_output_png_header(out, pixmap->w, pixmap->h, pixmap->n, savealpha);
 
-	fz_var(udata);
-	fz_var(cdata);
+	fz_try(ctx)
+	{
+		fz_output_png_band(out, pixmap->w, pixmap->h, pixmap->n, 0, pixmap->h, pixmap->samples, savealpha, poc);
+	}
+	fz_always(ctx)
+	{
+		fz_output_png_trailer(out, poc);
+	}
+	fz_catch(ctx)
+	{
+		fz_rethrow(ctx);
+	}
+}
 
-	if (pixmap->n != 1 && pixmap->n != 2 && pixmap->n != 4)
+struct fz_png_output_context_s
+{
+	unsigned char *udata;
+	unsigned char *cdata;
+	uLong usize, csize;
+	z_stream stream;
+};
+
+fz_png_output_context *
+fz_output_png_header(fz_output *out, int w, int h, int n, int savealpha)
+{
+	static const unsigned char pngsig[8] = { 137, 80, 78, 71, 13, 10, 26, 10 };
+	unsigned char head[13];
+	fz_context *ctx;
+	int color;
+	fz_png_output_context *poc;
+
+	if (!out)
+		return NULL;
+
+	ctx = out->ctx;
+
+	if (n != 1 && n != 2 && n != 4)
 		fz_throw(ctx, FZ_ERROR_GENERIC, "pixmap must be grayscale or rgb to write as png");
 
-	sn = pixmap->n;
-	dn = pixmap->n;
-	if (!savealpha && dn > 1)
-		dn--;
+	poc = fz_malloc_struct(ctx, fz_png_output_context);
 
-	switch (dn)
+	if (!savealpha && n > 1)
+		n--;
+
+	switch (n)
 	{
 	default:
 	case 1: color = 0; break;
@@ -625,25 +763,76 @@ fz_output_png(fz_output *out, const fz_pixmap *pixmap, int savealpha)
 	case 4: color = 6; break;
 	}
 
-	usize = (pixmap->w * dn + 1) * pixmap->h;
-	csize = compressBound(usize);
-	fz_try(ctx)
+	big32(head+0, w);
+	big32(head+4, h);
+	head[8] = 8; /* depth */
+	head[9] = color;
+	head[10] = 0; /* compression */
+	head[11] = 0; /* filter */
+	head[12] = 0; /* interlace */
+
+	fz_write(out, pngsig, 8);
+	putchunk("IHDR", head, 13, out);
+
+	return poc;
+}
+
+void
+fz_output_png_band(fz_output *out, int w, int h, int n, int band, int bandheight, unsigned char *sp, int savealpha, fz_png_output_context *poc)
+{
+	unsigned char *dp;
+	int y, x, k, sn, dn, err, finalband;
+	fz_context *ctx;
+
+	if (!out || !sp || !poc)
+		return;
+
+	ctx = out->ctx;
+
+	if (n != 1 && n != 2 && n != 4)
+		fz_throw(ctx, FZ_ERROR_GENERIC, "pixmap must be grayscale or rgb to write as png");
+
+	band *= bandheight;
+	finalband = (band+bandheight >= h);
+	if (finalband)
+		bandheight = h - band;
+
+	sn = n;
+	dn = n;
+	if (!savealpha && dn > 1)
+		dn--;
+
+	if (poc->udata == NULL)
 	{
-		udata = fz_malloc(ctx, usize);
-		cdata = fz_malloc(ctx, csize);
-	}
-	fz_catch(ctx)
-	{
-		fz_free(ctx, udata);
-		fz_rethrow(ctx);
+		poc->usize = (w * dn + 1) * bandheight;
+		/* Sadly the bound returned by compressBound is just for a
+		 * single usize chunk; if you compress a sequence of them
+		 * the buffering can result in you suddenly getting a block
+		 * larger than compressBound outputted in one go, even if you
+		 * take all the data out each time. */
+		poc->csize = compressBound(poc->usize);
+		fz_try(ctx)
+		{
+			poc->udata = fz_malloc(ctx, poc->usize);
+			poc->cdata = fz_malloc(ctx, poc->csize);
+		}
+		fz_catch(ctx)
+		{
+			fz_free(ctx, poc->udata);
+			poc->udata = NULL;
+			poc->cdata = NULL;
+			fz_rethrow(ctx);
+		}
+		err = deflateInit(&poc->stream, Z_DEFAULT_COMPRESSION);
+		if (err != Z_OK)
+			fz_throw(ctx, FZ_ERROR_GENERIC, "compression error %d", err);
 	}
 
-	sp = pixmap->samples;
-	dp = udata;
-	for (y = 0; y < pixmap->h; y++)
+	dp = poc->udata;
+	for (y = 0; y < bandheight; y++)
 	{
 		*dp++ = 1; /* sub prediction filter */
-		for (x = 0; x < pixmap->w; x++)
+		for (x = 0; x < w; x++)
 		{
 			for (k = 0; k < dn; k++)
 			{
@@ -657,58 +846,90 @@ fz_output_png(fz_output *out, const fz_pixmap *pixmap, int savealpha)
 		}
 	}
 
-	err = compress(cdata, &csize, udata, usize);
-	if (err != Z_OK)
+	poc->stream.next_in = (Bytef*)poc->udata;
+	poc->stream.avail_in = (uInt)(dp - poc->udata);
+	do
 	{
-		fz_free(ctx, udata);
-		fz_free(ctx, cdata);
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot compress image data");
+		poc->stream.next_out = poc->cdata;
+		poc->stream.avail_out = (uInt)poc->csize;
+
+		if (!finalband)
+		{
+			err = deflate(&poc->stream, Z_NO_FLUSH);
+			if (err != Z_OK)
+				fz_throw(ctx, FZ_ERROR_GENERIC, "compression error %d", err);
+		}
+		else
+		{
+			err = deflate(&poc->stream, Z_FINISH);
+			if (err != Z_STREAM_END)
+				fz_throw(ctx, FZ_ERROR_GENERIC, "compression error %d", err);
+		}
+
+		if (poc->stream.next_out != poc->cdata)
+			putchunk("IDAT", poc->cdata, poc->stream.next_out - poc->cdata, out);
 	}
-
-	big32(head+0, pixmap->w);
-	big32(head+4, pixmap->h);
-	head[8] = 8; /* depth */
-	head[9] = color;
-	head[10] = 0; /* compression */
-	head[11] = 0; /* filter */
-	head[12] = 0; /* interlace */
-
-	fz_write(out, pngsig, 8);
-	putchunk("IHDR", head, 13, out);
-	putchunk("IDAT", cdata, csize, out);
-	putchunk("IEND", head, 0, out);
-
-	fz_free(ctx, udata);
-	fz_free(ctx, cdata);
+	while (poc->stream.avail_out == 0);
 }
 
-fz_buffer *
-fz_image_as_png(fz_context *ctx, fz_image *image, int w, int h)
+void
+fz_output_png_trailer(fz_output *out, fz_png_output_context *poc)
+{
+	unsigned char block[1];
+	int err;
+	fz_context *ctx;
+
+	if (!out || !poc)
+		return;
+
+	ctx = out->ctx;
+
+	err = deflateEnd(&poc->stream);
+	if (err != Z_OK)
+		fz_throw(ctx, FZ_ERROR_GENERIC, "compression error %d", err);
+
+	fz_free(ctx, poc->cdata);
+	fz_free(ctx, poc->udata);
+	fz_free(ctx, poc);
+
+	putchunk("IEND", block, 0, out);
+}
+
+/* We use an auxiliary function to do pixmap_as_png, as it can enable us to
+ * drop pix early in the case where we have to convert, potentially saving
+ * us having to have 2 copies of the pixmap and a buffer open at once. */
+static fz_buffer *
+png_from_pixmap(fz_context *ctx, fz_pixmap *pix, int drop)
 {
-	fz_pixmap *pix = fz_image_get_pixmap(ctx, image, image->w, image->h);
 	fz_buffer *buf = NULL;
 	fz_output *out;
+	fz_pixmap *pix2 = NULL;
 
 	fz_var(buf);
 	fz_var(out);
+	fz_var(pix2);
+
+	if (pix->w == 0 || pix->h == 0)
+		return NULL;
 
 	fz_try(ctx)
 	{
-		if (pix->colorspace != fz_device_gray(ctx) || pix->colorspace != fz_device_rgb(ctx))
+		if (pix->colorspace && pix->colorspace != fz_device_gray(ctx) && pix->colorspace != fz_device_rgb(ctx))
 		{
-			fz_pixmap *pix2 = fz_new_pixmap(ctx, fz_device_rgb(ctx), pix->w, pix->h);
+			pix2 = fz_new_pixmap(ctx, fz_device_rgb(ctx), pix->w, pix->h);
 			fz_convert_pixmap(ctx, pix2, pix);
-			fz_drop_pixmap(ctx, pix);
+			if (drop)
+				fz_drop_pixmap(ctx, pix);
 			pix = pix2;
 		}
 		buf = fz_new_buffer(ctx, 1024);
 		out = fz_new_output_with_buffer(ctx, buf);
-		fz_output_png(out, pix, 0);
+		fz_output_png(out, pix, 1);
 	}
 	fz_always(ctx)
 	{
+		fz_drop_pixmap(ctx, drop ? pix : pix2);
 		fz_close_output(out);
-		fz_drop_pixmap(ctx, pix);
 	}
 	fz_catch(ctx)
 	{
@@ -718,6 +939,99 @@ fz_image_as_png(fz_context *ctx, fz_image *image, int w, int h)
 	return buf;
 }
 
+fz_buffer *
+fz_new_png_from_image(fz_context *ctx, fz_image *image, int w, int h)
+{
+	fz_pixmap *pix = fz_image_get_pixmap(ctx, image, image->w, image->h);
+
+	return png_from_pixmap(ctx, pix, 1);
+}
+
+fz_buffer *
+fz_new_png_from_pixmap(fz_context *ctx, fz_pixmap *pix)
+{
+	return png_from_pixmap(ctx, pix, 0);
+}
+
+/*
+ * Write pixmap to TGA file (with or without alpha channel)
+ */
+
+static inline void tga_put_pixel(unsigned char *data, int n, int is_bgr, fz_output *out)
+{
+	if (n >= 3 && !is_bgr)
+	{
+		fz_putc(out, data[2]);
+		fz_putc(out, data[1]);
+		fz_putc(out, data[0]);
+		if (n == 4)
+			fz_putc(out, data[3]);
+		return;
+	}
+	if (n == 2)
+	{
+		fz_putc(out, data[0]);
+		fz_putc(out, data[0]);
+	}
+	fz_write(out, data, n);
+}
+
+void
+fz_write_tga(fz_context *ctx, fz_pixmap *pixmap, const char *filename, int savealpha)
+{
+	fz_output *out;
+	unsigned char head[18];
+	int n = pixmap->n;
+	int d = savealpha || n == 1 ? n : n - 1;
+	int is_bgr = pixmap->colorspace == fz_device_bgr(ctx);
+	int k;
+
+	if (pixmap->colorspace && pixmap->colorspace != fz_device_gray(ctx) &&
+		pixmap->colorspace != fz_device_rgb(ctx) && pixmap->colorspace != fz_device_bgr(ctx))
+	{
+		fz_throw(ctx, FZ_ERROR_GENERIC, "pixmap must be grayscale or rgb to write as tga");
+	}
+
+	out = fz_new_output_to_filename(ctx, filename);
+
+	memset(head, 0, sizeof(head));
+	head[2] = n == 4 ? 10 : 11;
+	head[12] = pixmap->w & 0xFF; head[13] = (pixmap->w >> 8) & 0xFF;
+	head[14] = pixmap->h & 0xFF; head[15] = (pixmap->h >> 8) & 0xFF;
+	head[16] = d * 8;
+	head[17] = savealpha && n > 1 ? 8 : 0;
+	if (savealpha && d == 2)
+		head[16] = 32;
+
+	fz_write(out, head, sizeof(head));
+	for (k = 1; k <= pixmap->h; k++)
+	{
+		int i, j;
+		unsigned char *line = pixmap->samples + pixmap->w * n * (pixmap->h - k);
+		for (i = 0, j = 1; i < pixmap->w; i += j, j = 1)
+		{
+			for (; i + j < pixmap->w && j < 128 && !memcmp(line + i * n, line + (i + j) * n, d); j++);
+			if (j > 1)
+			{
+				fz_putc(out, j - 1 + 128);
+				tga_put_pixel(line + i * n, d, is_bgr, out);
+			}
+			else
+			{
+				for (; i + j < pixmap->w && j <= 128 && memcmp(line + (i + j - 1) * n, line + (i + j) * n, d) != 0; j++);
+				if (i + j < pixmap->w || j > 128)
+					j--;
+				fz_putc(out, j - 1);
+				for (; j > 0; j--, i++)
+					tga_put_pixel(line + i * n, d, is_bgr, out);
+			}
+		}
+	}
+	fz_write(out, "\0\0\0\0\0\0\0\0TRUEVISION-XFILE.\0", 26);
+
+	fz_close_output(out);
+}
+
 unsigned int
 fz_pixmap_size(fz_context *ctx, fz_pixmap * pix)
 {
@@ -726,6 +1040,44 @@ fz_pixmap_size(fz_context *ctx, fz_pixmap * pix)
 	return sizeof(*pix) + pix->n * pix->w * pix->h;
 }
 
+fz_pixmap *
+fz_new_pixmap_from_8bpp_data(fz_context *ctx, int x, int y, int w, int h, unsigned char *sp, int span)
+{
+	fz_pixmap *pixmap = fz_new_pixmap(ctx, NULL, w, h);
+	pixmap->x = x;
+	pixmap->y = y;
+
+	for (y = 0; y < h; y++)
+		memcpy(pixmap->samples + y * w, sp + y * span, w);
+
+	return pixmap;
+}
+
+fz_pixmap *
+fz_new_pixmap_from_1bpp_data(fz_context *ctx, int x, int y, int w, int h, unsigned char *sp, int span)
+{
+	fz_pixmap *pixmap = fz_new_pixmap(ctx, NULL, w, h);
+	pixmap->x = x;
+	pixmap->y = y;
+
+	for (y = 0; y < h; y++)
+	{
+		unsigned char *out = pixmap->samples + y * w;
+		unsigned char *in = sp + y * span;
+		unsigned char bit = 0x80;
+		int ww = w;
+		while (ww--)
+		{
+			*out++ = (*in & bit) ? 255 : 0;
+			bit >>= 1;
+			if (bit == 0)
+				bit = 0x80, in++;
+		}
+	}
+
+	return pixmap;
+}
+
 #ifdef ARCH_ARM
 static void
 fz_subsample_pixmap_ARM(unsigned char *ptr, int w, int h, int f, int factor,
