@@ -1,18 +1,18 @@
 #include "mupdf/pdf.h"
 
-pdf_js *pdf_new_js(pdf_document *doc)
-{
-	return NULL;
-}
-
-void pdf_js_load_document_level(pdf_js *js)
+void pdf_enable_js(pdf_document *doc)
 {
 }
 
-void pdf_drop_js(pdf_js *js)
+void pdf_disable_js(pdf_document *doc)
 {
 }
 
+int pdf_js_supported(pdf_document *doc)
+{
+	return 0;
+}
+
 void pdf_js_setup_event(pdf_js *js, pdf_js_event *e)
 {
 }
@@ -29,8 +29,3 @@ void pdf_js_execute(pdf_js *js, char *code)
 void pdf_js_execute_count(pdf_js *js, char *code, int count)
 {
 }
-
-int pdf_js_supported(void)
-{
-	return 0;
-}
