@@ -1,68 +1,63 @@
-var MuPDF = new Array();
+var MuPDF = {
+	monthName: ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'],
+	shortMonthName: ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'],
+	monthPattern: /Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec/,
+	dayName: ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'],
 
-MuPDF.monthName = ['January','February','March','April','May','June','July','August','September','October','November','December'];
-MuPDF.dayName = ['Sunday','Monday','Tuesday','Wednesday','Thursday','Friday','Saturday'];
+	padZeros: function(num, places) {
+		var s = String(num)
+		while (s.length < places)
+			s = '0' + s
+		return s;
+	},
 
-MuPDF.shortMonthName = new Array();
-
-for (var i = 0; i < MuPDF.monthName.length; i++)
-	MuPDF.shortMonthName.push(MuPDF.monthName[i].substr(0,3));
-
-MuPDF.monthPattern = new RegExp();
-MuPDF.monthPattern.compile('('+MuPDF.shortMonthName.join('|')+')');
-
-MuPDF.padZeros = function(num, places)
-{
-	var s = num.toString();
-
-	if (s.length < places)
-		s = new Array(places-s.length+1).join('0') + s;
-
-	return s;
-}
-
-MuPDF.convertCase = function(str,cmd)
-{
-	switch (cmd)
-	{
-		case '>': return str.toUpperCase();
-		case '<': return str.toLowerCase();
-		default: return str;
-	}
-}
+	convertCase: function(str, cmd) {
+		switch (cmd) {
+			case '>': return str.toUpperCase();
+			case '<': return str.toLowerCase();
+			default: return str;
+		}
+	},
+};
 
 /* display must be kept in sync with an enum in pdf_form.c */
-var display = new Array();
-display.visible = 0;
-display.hidden = 1;
-display.noPrint = 2;
-display.noView = 3;
-var border = new Array();
-border.s = "Solid";
-border.d = "Dashed";
-border.b = "Beveled";
-border.i = "Inset";
-border.u = "Underline";
-var color = new Array();
-color.transparent = [ "T" ];
-color.black = [ "G", 0];
-color.white = [ "G", 1];
-color.red = [ "RGB", 1,0,0 ];
-color.green = [ "RGB", 0,1,0 ];
-color.blue = [ "RGB", 0,0,1 ];
-color.cyan = [ "CMYK", 1,0,0,0 ];
-color.magenta = [ "CMYK", 0,1,0,0 ];
-color.yellow = [ "CMYK", 0,0,1,0 ];
-color.dkGray = [ "G", 0.25];
-color.gray = [ "G", 0.5];
-color.ltGray = [ "G", 0.75];
+var display = {
+	visible: 0,
+	hidden: 1,
+	noPrint: 2,
+	noView: 3,
+};
 
-var util = new Array();
+var border = {
+	s: "Solid",
+	d: "Dashed",
+	b: "Beveled",
+	i: "Inset",
+	u: "Underline",
+};
+
+var color = {
+	transparent: [ "T" ],
+	black: [ "G", 0 ],
+	white: [ "G", 1 ],
+	red: [ "RGB", 1,0,0 ],
+	green: [ "RGB", 0,1,0 ],
+	blue: [ "RGB", 0,0,1 ],
+	cyan: [ "CMYK", 1,0,0,0 ],
+	magenta: [ "CMYK", 0,1,0,0 ],
+	yellow: [ "CMYK", 0,0,1,0 ],
+	dkGray: [ "G", 0.25 ],
+	gray: [ "G", 0.5 ],
+	ltGray: [ "G", 0.75 ],
+};
+
+var util = {};
 
 util.printd = function(fmt, d)
 {
 	var regexp = /(m+|d+|y+|H+|h+|M+|s+|t+|[^mdyHhMst]+)/g;
 	var res = '';
+	var i;
 
 	if (!d)
 		return null;
@@ -70,16 +65,16 @@ util.printd = function(fmt, d)
 	var tokens = fmt.match(regexp);
 	var length = tokens ? tokens.length : 0;
 
-	for (var i = 0; i < length; i++)
+	for (i = 0; i < length; i++)
 	{
 		switch(tokens[i])
 		{
 			case 'mmmm': res += MuPDF.monthName[d.getMonth()]; break;
-			case 'mmm': res += MuPDF.monthName[d.getMonth()].substr(0,3); break;
+			case 'mmm': res += MuPDF.monthName[d.getMonth()].substring(0,3); break;
 			case 'mm': res += MuPDF.padZeros(d.getMonth()+1, 2); break;
 			case 'm': res += d.getMonth()+1; break;
 			case 'dddd': res += MuPDF.dayName[d.getDay()]; break;
-			case 'ddd': res += MuPDF.dayName[d.getDay()].substr(0,3); break;
+			case 'ddd': res += MuPDF.dayName[d.getDay()].substring(0,3); break;
 			case 'dd': res += MuPDF.padZeros(d.getDate(), 2); break;
 			case 'd': res += d.getDate(); break;
 			case 'yyyy': res += d.getFullYear(); break;
@@ -127,10 +122,10 @@ util.printx = function(fmt, val)
 				break;
 
 			case 'A':
-				m = val.match(/[A-z]/);
+				m = val.match(/[A-Za-z]/);
 				if (!m) return res;
 				res += MuPDF.convertCase(m[0],cs);
-				val = val.replace(/^[^A-z]*[A-z]/,'');
+				val = val.replace(/^[^A-Za-z]*[A-Za-z]/,'');
 				break;
 
 			case '9':
@@ -148,7 +143,7 @@ util.printx = function(fmt, val)
 			case '?':
 				if (!val) return res;
 				res += MuPDF.convertCase(val.charAt(0),cs);
-				val = val.substr(1);
+				val = val.substring(1);
 				break;
 
 			case '=':
@@ -170,6 +165,8 @@ util.printx = function(fmt, val)
 
 util.printf = function()
 {
+	var i;
+
 	if (arguments.length < 1)
 		return "";
 
@@ -179,7 +176,7 @@ util.printf = function()
 	var tokens = arguments[0].match(regexp);
 	var length = tokens ? tokens.length : 0;
 
-	for (var i = 0; i < length; i++)
+	for (i = 0; i < length; i++)
 	{
 		var tok = tokens[i];
 		if (tok.match(/^%/))
@@ -272,7 +269,7 @@ util.printf = function()
 								// Matched string includes the dot, so make it
 								// prec+1 in length
 								if (frac.length > prec+1)
-									frac = frac.substr(0, prec+1);
+									frac = frac.substring(0, prec+1);
 								else if(frac.length < prec+1)
 									frac += new Array(prec+1-frac.length+1).join('0');
 
@@ -287,7 +284,7 @@ util.printf = function()
 					if (fval >= 0)
 						fval = poschar + fval;
 
-					if (point != '.')
+					if (point !== '.')
 						fval.replace(/\./, point);
 
 					if (thou)
@@ -351,16 +348,17 @@ function AFExtractTime(dt)
 
 function AFParseDateOrder(fmt)
 {
+	var i;
 	var order = '';
 
 	// Ensure all present with those not added in default order
 	fmt += "mdy";
 
-	for (var i = 0; i < fmt.length; i++)
+	for (i = 0; i < fmt.length; i++)
 	{
 		var c = fmt.charAt(i);
 
-		if ('ymd'.indexOf(c) != -1 && order.indexOf(c) == -1)
+		if ('ymd'.indexOf(c) !== -1 && order.indexOf(c) === -1)
 			order += c;
 	}
 
@@ -394,25 +392,26 @@ function AFParseTime(str, d)
 	switch (nums.length)
 	{
 		case 3:
-			sec = nums[2];
+			sec = parseInt(nums[2]);
 		case 2:
-			hour = nums[0];
-			min = nums[1];
+			hour = parseInt(nums[0]);
+			min = parseInt(nums[1]);
 			break;
 
 		default:
 			return null;
 	}
 
-	if (ampm == 'am' && hour < 12)
-		hour = 12 + hour;
+	ampm = ampm && ampm[0]
 
-	if (ampm == 'pm' && hour >= 12)
+	if (ampm === 'am' && hour < 12)
+		hour = 12 + hour;
+	if (ampm === 'pm' && hour >= 12)
 		hour = 0 + hour - 12;
 
 	d.setHours(hour, min, sec);
 
-	if (d.getHours() != hour || d.getMinutes() != min || d.getSeconds() != sec)
+	if (d.getHours() !== hour || d.getMinutes() !== min || d.getSeconds() !== sec)
 		return null;
 
 	return d;
@@ -420,6 +419,7 @@ function AFParseTime(str, d)
 
 function AFParseDateEx(d, fmt)
 {
+	var i;
 	var dt = AFExtractTime(d);
 	var nums = dt[0].match(/\d+/g);
 	var order = AFParseDateOrder(fmt);
@@ -441,25 +441,25 @@ function AFParseDateEx(d, fmt)
 		order = order.replace('m','');
 	}
 
-	order = order.substr(0, nums.length);
+	order = order.substring(0, nums.length);
 
 	// If year and month specified but not date then use the 1st
-	if (order == "ym" || (order == "y" && text_month))
+	if (order === "ym" || (order === "y" && text_month))
 		date = 1;
 
-	for (var i = 0; i < nums.length; i++)
+	for (i = 0; i < nums.length; i++)
 	{
 		switch (order.charAt(i))
 		{
-			case 'y': year = nums[i]; break;
-			case 'm': month = nums[i] - 1; break;
-			case 'd': date = nums[i]; break;
+			case 'y': year = parseInt(nums[i]); break;
+			case 'm': month = parseInt(nums[i]) - 1; break;
+			case 'd': date = parseInt(nums[i]); break;
 		}
 	}
 
 	if (year < 100)
 	{
-		if (fmt.search("yyyy") != -1)
+		if (fmt.search("yyyy") !== -1)
 			return null;
 
 		if (year >= 50)
@@ -470,7 +470,7 @@ function AFParseDateEx(d, fmt)
 
 	dout.setFullYear(year, month, date);
 
-	if (dout.getFullYear() != year || dout.getMonth() != month || dout.getDate() != date)
+	if (dout.getFullYear() !== year || dout.getMonth() !== month || dout.getDate() !== date)
 		return null;
 
 	return AFParseTime(dt[1], dout);
@@ -480,7 +480,7 @@ function AFDate_KeystrokeEx(fmt)
 {
 	if (event.willCommit && !AFParseDateEx(event.value, fmt))
 	{
-		app.alert("Invalid date/time. please ensure that the date/time exists. Field [ "+event.target.name+" ] should match format "+fmt);
+		app.alert("The date/time entered ("+event.value+") does not match the format ("+fmt+") of the field [ "+event.target.name+" ]");
 		event.rc = false;
 	}
 }
@@ -510,7 +510,7 @@ function AFTime_Keystroke(index)
 {
 	if (event.willCommit && !AFParseTime(event.value, null))
 	{
-		app.alert("The value entered does not match the format of the field [ "+event.target.name+" ]");
+		app.alert("The value entered ("+event.value+") does not match the format of the field [ "+event.target.name+" ]");
 		event.rc = false;
 	}
 }
@@ -547,8 +547,8 @@ function AFSpecial_KeystrokeEx(fmt)
 				if (i >= length)
 					break;
 				res += fmt.charAt(i);
-				if (val && val.charAt(0) == fmt.charAt(i))
-					val = val.substr(1);
+				if (val && val.charAt(0) === fmt.charAt(i))
+					val = val.substring(1);
 				break;
 
 			case 'X':
@@ -559,18 +559,18 @@ function AFSpecial_KeystrokeEx(fmt)
 					break;
 				}
 				res += MuPDF.convertCase(m[0],cs);
-				val = val.substr(1);
+				val = val.substring(1);
 				break;
 
 			case 'A':
-				m = val.match(/^[A-z]/);
+				m = val.match(/^[A-Za-z]/);
 				if (!m)
 				{
 					event.rc = false;
 					break;
 				}
 				res += MuPDF.convertCase(m[0],cs);
-				val = val.substr(1);
+				val = val.substring(1);
 				break;
 
 			case '9':
@@ -581,7 +581,7 @@ function AFSpecial_KeystrokeEx(fmt)
 					break;
 				}
 				res += m[0];
-				val = val.substr(1);
+				val = val.substring(1);
 				break;
 
 			case '*':
@@ -596,7 +596,7 @@ function AFSpecial_KeystrokeEx(fmt)
 					break;
 				}
 				res += MuPDF.convertCase(val.charAt(0),cs);
-				val = val.substr(1);
+				val = val.substring(1);
 				break;
 
 			case '=':
@@ -607,8 +607,8 @@ function AFSpecial_KeystrokeEx(fmt)
 
 			default:
 				res += fmt.charAt(i);
-				if (val && val.charAt(0) == fmt.charAt(i))
-					val = val.substr(1);
+				if (val && val.charAt(0) === fmt.charAt(i))
+					val = val.substring(1);
 				break;
 		}
 
@@ -618,7 +618,7 @@ function AFSpecial_KeystrokeEx(fmt)
 	if (event.rc)
 		event.value = res;
 	else if (event.willCommit)
-		app.alert("The value entered does not match the format of the field [ "+event.target.name+" ] should be "+fmt);
+		app.alert("The value entered ("+event.value+") does not match the format of the field [ "+event.target.name+" ] should be "+fmt);
 }
 
 function AFSpecial_Keystroke(index)
@@ -646,7 +646,7 @@ function AFSpecial_Keystroke(index)
 		}
 
 		if (!event.rc)
-			app.alert("The value entered does not match the format of the field [ "+event.target.name+" ]");
+			app.alert("The value entered ("+event.value+") does not match the format of the field [ "+event.target.name+" ]");
 	}
 }
 
@@ -693,7 +693,7 @@ function AFNumber_Keystroke(nDec, sepStyle, negStyle, currStyle, strCurrency, bC
 			event.rc = false;
 
 		if (!event.rc)
-			app.alert("The value entered does not match the format of the field [ "+event.target.name+" ]");
+			app.alert("The value entered ("+event.value+") does not match the format of the field [ "+event.target.name+" ]");
 	}
 }
 
@@ -715,16 +715,16 @@ function AFNumber_Format(nDec,sepStyle,negStyle,currStyle,strCurrency,bCurrencyP
 
 	switch (groups.length)
 	{
-	case 0:
-		return;
-	case 1:
-		fracpart = '';
-		intpart = groups[0];
-		break;
-	default:
-		fracpart = groups.pop();
-		intpart = groups.join('');
-		break;
+		case 0:
+			return;
+		case 1:
+			fracpart = '';
+			intpart = groups[0];
+			break;
+		default:
+			fracpart = groups.pop();
+			intpart = groups.join('');
+			break;
 	}
 
 	// Remove leading zeros
@@ -732,7 +732,7 @@ function AFNumber_Format(nDec,sepStyle,negStyle,currStyle,strCurrency,bCurrencyP
 	if (!intpart)
 		intpart = '0';
 
-	if ((sepStyle & 1) == 0)
+	if ((sepStyle & 1) === 0)
 	{
 		// Add the thousands sepearators: pad to length multiple of 3 with zeros,
 		// split into 3s, join with separator, and remove the leading zeros
@@ -745,7 +745,7 @@ function AFNumber_Format(nDec,sepStyle,negStyle,currStyle,strCurrency,bCurrencyP
 
 	// Adjust fractional part to correct number of decimal places
 	fracpart += new Array(nDec+1).join('0');
-	fracpart = fracpart.substr(0,nDec);
+	fracpart = fracpart.substring(0,nDec);
 
 	if (fracpart)
 		intpart += point+fracpart;
@@ -759,15 +759,15 @@ function AFNumber_Format(nDec,sepStyle,negStyle,currStyle,strCurrency,bCurrencyP
 	{
 		switch (negStyle)
 		{
-		case 0:
-			intpart = '-'+intpart;
-			break;
-		case 1:
-			break;
-		case 2:
-		case 3:
-			intpart = '('+intpart+')';
-			break;
+			case 0:
+				intpart = '-'+intpart;
+				break;
+			case 1:
+				break;
+			case 2:
+			case 3:
+				intpart = '('+intpart+')';
+				break;
 		}
 	}
 
@@ -799,7 +799,7 @@ function AFPercent_Format(nDec, sepStyle)
 
 function AFSimple_Calculate(op, list)
 {
-	var res;
+	var i, res;
 
 	switch (op)
 	{
@@ -814,10 +814,10 @@ function AFSimple_Calculate(op, list)
 			break;
 	}
 
-	if (typeof list == 'string')
+	if (typeof list === 'string')
 		list = list.split(/ *, */);
 
-	for (var i = 0; i < list.length; i++)
+	for (i = 0; i < list.length; i++)
 	{
 		var field = getField(list[i]);
 		var value = Number(field.value);
@@ -834,17 +834,17 @@ function AFSimple_Calculate(op, list)
 				res += value;
 				break;
 			case 'MIN':
-				if (i == 0 || value < res)
+				if (i === 0 || value < res)
 					res = value;
 				break;
 			case 'MAX':
-				if (i == 0 || value > res)
+				if (i === 0 || value > res)
 					res = value;
 				break;
 		}
 	}
 
-	if (op == 'AVG')
+	if (op === 'AVG')
 		res /= list.length;
 
 	event.value = res;
@@ -865,10 +865,10 @@ function AFRange_Validate(lowerCheck, lowerLimit, upperCheck, upperLimit)
 	if (!event.rc)
 	{
 		if (lowerCheck && upperCheck)
-			app.alert(util.printf("Invalid value: must be greater than or equal to %s and less than or equal to %s", lowerLimit, upperLimit));
+			app.alert(util.printf("The entered value ("+event.value+") must be greater than or equal to %s and less than or equal to %s", lowerLimit, upperLimit));
 		else if (lowerCheck)
-			app.alert(util.printf("Invalid value: must be greater than or equal to %s", lowerLimit));
+			app.alert(util.printf("The entered value ("+event.value+") must be greater than or equal to %s", lowerLimit));
 		else
-			app.alert(util.printf("Invalid value: must be less than or equal to %s", upperLimit));
+			app.alert(util.printf("The entered value ("+event.value+") must be less than or equal to %s", upperLimit));
 	}
 }
