@@ -1,5 +1,9 @@
 #include "mupdf/pdf.h"
 
+#include <ft2build.h>
+#include FT_FREETYPE_H
+#include FT_ADVANCES_H
+
 #define MATRIX_COEFS (6)
 #define STRIKE_HEIGHT (0.375f)
 #define UNDERLINE_HEIGHT (0.075f)
@@ -26,6 +30,7 @@ typedef struct font_info_s
 {
 	pdf_da_info da_rec;
 	pdf_font_desc *font;
+	float lineheight;
 } font_info;
 
 typedef struct text_widget_info_s
@@ -51,7 +56,7 @@ static const char *fmt_q = "q\n";
 static const char *fmt_W = "W\n";
 static const char *fmt_n = "n\n";
 static const char *fmt_BT = "BT\n";
-static const char *fmt_Tm = "%1.2f %1.2f %1.2f %1.2f %1.2f %1.2f Tm\n";
+static const char *fmt_Tm = "%f %f %f %f %f %f Tm\n";
 static const char *fmt_Td = "%f %f Td\n";
 static const char *fmt_Tj = " Tj\n";
 static const char *fmt_ET = "ET\n";
@@ -75,7 +80,7 @@ static void da_check_stack(float *stack, int *top)
 
 void pdf_parse_da(fz_context *ctx, char *da, pdf_da_info *di)
 {
-	float stack[32];
+	float stack[32] = { 0.0f };
 	int top = 0;
 	pdf_token tok;
 	char *name = NULL;
@@ -123,6 +128,11 @@ void pdf_parse_da(fz_context *ctx, char *da, pdf_da_info *di)
 					di->col[2] = stack[2];
 					di->col_size = 3;
 				}
+				else if (!strcmp(lbuf.scratch, "g"))
+				{
+					di->col[0] = stack[0];
+					di->col_size = 1;
+				}
 
 				fz_free(ctx, name);
 				name = NULL;
@@ -149,11 +159,16 @@ void pdf_parse_da(fz_context *ctx, char *da, pdf_da_info *di)
 static void get_font_info(pdf_document *doc, pdf_obj *dr, char *da, font_info *font_rec)
 {
 	fz_context *ctx = doc->ctx;
+	pdf_font_desc *font;
 
 	pdf_parse_da(ctx, da, &font_rec->da_rec);
 	if (font_rec->da_rec.font_name == NULL)
 		fz_throw(ctx, FZ_ERROR_GENERIC, "No font name in default appearance");
-	font_rec->font = pdf_load_font(doc, dr, pdf_dict_gets(pdf_dict_gets(dr, "Font"), font_rec->da_rec.font_name), 0);
+
+	font_rec->font = font = pdf_load_font(doc, dr, pdf_dict_gets(pdf_dict_gets(dr, "Font"), font_rec->da_rec.font_name), 0);
+	font_rec->lineheight = 1.0;
+	if (font && font->ascent != 0.0f && font->descent != 0.0f)
+		font_rec->lineheight = (font->ascent - font->descent) / 1000.0;
 }
 
 static void font_info_fin(fz_context *ctx, font_info *font_rec)
@@ -349,6 +364,7 @@ typedef struct text_splitter_s
 	float scale;
 	float unscaled_width;
 	float fontsize;
+	float lineheight;
 	char *text;
 	int done;
 	float x_orig;
@@ -373,10 +389,11 @@ static void text_splitter_init(text_splitter *splitter, font_info *info, char *t
 	splitter->height = height;
 	splitter->fontsize = fontsize;
 	splitter->scale = 1.0;
+	splitter->lineheight = fontsize * info->lineheight ;
 	/* RJW: The cast in the following line is important, as otherwise
 	 * under MSVC in the variable = 0 case, splitter->max_lines becomes
 	 * INT_MIN. */
-	splitter->max_lines = variable ? (int)(height/fontsize) : INT_MAX;
+	splitter->max_lines = variable ? (int)(height/splitter->lineheight) : INT_MAX;
 }
 
 static void text_splitter_start_pass(text_splitter *splitter)
@@ -451,7 +468,7 @@ static int text_splitter_layout(fz_context *ctx, text_splitter *splitter)
 			? splitter->width * 1.1 / splitter->scale
 			: FLT_MAX;
 
-		vstretchwidth = splitter->width * (splitter->max_lines + 1) * splitter->fontsize
+		vstretchwidth = splitter->width * (splitter->max_lines + 1) * splitter->lineheight
 			/ splitter->height;
 
 		bestwidth = fz_min(fitwidth, fz_min(hstretchwidth, vstretchwidth));
@@ -483,10 +500,10 @@ static int text_splitter_layout(fz_context *ctx, text_splitter *splitter)
 static void text_splitter_move(text_splitter *splitter, float newy, float *relx, float *rely)
 {
 	*relx = splitter->x - splitter->x_orig;
-	*rely = newy - splitter->y_orig;
+	*rely = newy * splitter->lineheight - splitter->y_orig;
 
 	splitter->x_orig = splitter->x;
-	splitter->y_orig = newy;
+	splitter->y_orig = newy * splitter->lineheight;
 }
 
 static void text_splitter_retry(text_splitter *splitter)
@@ -497,7 +514,7 @@ static void text_splitter_retry(text_splitter *splitter)
 		 * be caused by carriage control */
 		splitter->max_lines ++;
 		splitter->retry = 0;
-		splitter->unscaled_width = splitter->width * splitter->max_lines * splitter->fontsize
+		splitter->unscaled_width = splitter->width * splitter->max_lines * splitter->lineheight
 			/ splitter->height;
 		splitter->scale = splitter->width / splitter->unscaled_width;
 	}
@@ -588,7 +605,7 @@ static fz_buffer *create_text_appearance(pdf_document *doc, const fz_rect *bbox,
 
 		variable = (info->font_rec.da_rec.font_size == 0);
 		fontsize = variable
-			? (info->multiline ? 14.0 : floor(height))
+			? (info->multiline ? 14.0 : height / info->font_rec.lineheight)
 			: info->font_rec.da_rec.font_size;
 
 		info->font_rec.da_rec.font_size = fontsize;
@@ -628,7 +645,7 @@ static fz_buffer *create_text_appearance(pdf_document *doc, const fz_rect *bbox,
 							char *word = text+splitter.text_start;
 							int wordlen = splitter.text_end-splitter.text_start;
 
-							text_splitter_move(&splitter, -line*fontsize, &x, &y);
+							text_splitter_move(&splitter, -line, &x, &y);
 							fzbuf_print_text_word(ctx, fztmp, x, y, word, wordlen);
 						}
 					}
@@ -1363,7 +1380,8 @@ void pdf_set_annot_appearance(pdf_document *doc, pdf_annot *annot, fz_rect *rect
 		else
 		{
 			pdf_xref_ensure_incremental_object(doc, pdf_to_num(ap_obj));
-			pdf_dict_puts_drop(ap_obj, "Rect", pdf_new_rect(doc, &trect));
+			/* Update bounding box and matrix in reused xobject obj */
+			pdf_dict_puts_drop(ap_obj, "BBox", pdf_new_rect(doc, &trect));
 			pdf_dict_puts_drop(ap_obj, "Matrix", pdf_new_matrix(doc, &mat));
 		}
 
@@ -1375,6 +1393,9 @@ void pdf_set_annot_appearance(pdf_document *doc, pdf_annot *annot, fz_rect *rect
 		xobj = pdf_load_xobject(doc, ap_obj);
 		if (xobj)
 		{
+			/* Update bounding box and matrix also in the xobject structure */
+			xobj->bbox = trect;
+			xobj->matrix = mat;
 			xobj->iteration++;
 			pdf_drop_xobject(ctx, xobj);
 		}
@@ -1537,7 +1558,7 @@ static fz_colorspace *pdf_to_color(pdf_document *doc, pdf_obj *col, float color[
 	return cs;
 }
 
-void pdf_set_ink_appearance(pdf_document *doc, pdf_annot *annot)
+void pdf_update_ink_appearance(pdf_document *doc, pdf_annot *annot)
 {
 	fz_context *ctx = doc->ctx;
 	const fz_matrix *page_ctm = &annot->page->ctm;
@@ -1545,19 +1566,21 @@ void pdf_set_ink_appearance(pdf_document *doc, pdf_annot *annot)
 	fz_stroke_state *stroke = NULL;
 	fz_device *dev = NULL;
 	fz_display_list *strike_list = NULL;
+	fz_colorspace *cs = NULL;
 
 	fz_var(path);
 	fz_var(stroke);
 	fz_var(dev);
 	fz_var(strike_list);
+	fz_var(cs);
 	fz_try(ctx)
 	{
 		fz_rect rect = fz_empty_rect;
-		fz_colorspace *cs;
 		float color[4];
 		float width;
 		pdf_obj *list;
 		int n, m, i, j;
+		int empty = 1;
 
 		cs = pdf_to_color(doc, pdf_dict_gets(annot->obj, "C"), color);
 		if (!cs)
@@ -1581,6 +1604,8 @@ void pdf_set_ink_appearance(pdf_document *doc, pdf_annot *annot)
 		path = fz_new_path(ctx);
 		stroke = fz_new_stroke_state(ctx);
 		stroke->linewidth = width;
+		stroke->start_cap = stroke->end_cap = FZ_LINECAP_ROUND;
+		stroke->linejoin = FZ_LINEJOIN_ROUND;
 
 		for (i = 0; i < n; i ++)
 		{
@@ -1598,6 +1623,7 @@ void pdf_set_ink_appearance(pdf_document *doc, pdf_annot *annot)
 				{
 					rect.x0 = rect.x1 = pt.x;
 					rect.y0 = rect.y1 = pt.y;
+					empty = 0;
 				}
 				else
 				{
@@ -1616,12 +1642,26 @@ void pdf_set_ink_appearance(pdf_document *doc, pdf_annot *annot)
 		fz_stroke_path(dev, path, stroke, page_ctm, cs, color, 1.0f);
 
 		fz_expand_rect(&rect, width);
+		/*
+			Expand the rectangle by width all around. We cannot use
+			fz_expand_rect because the rectangle might be empty in the
+			single point case
+		*/
+		if (!empty)
+		{
+			rect.x0 -= width;
+			rect.y0 -= width;
+			rect.x1 += width;
+			rect.y1 += width;
+		}
+
 
 		fz_transform_rect(&rect, page_ctm);
 		pdf_set_annot_appearance(doc, annot, &rect, strike_list);
 	}
 	fz_always(ctx)
 	{
+		fz_drop_colorspace(ctx, cs);
 		fz_free_device(dev);
 		fz_drop_stroke_state(ctx, stroke);
 		fz_free_path(ctx, path);
@@ -1632,3 +1672,618 @@ void pdf_set_ink_appearance(pdf_document *doc, pdf_annot *annot)
 		fz_rethrow(ctx);
 	}
 }
+
+static void add_text(fz_context *ctx, font_info *font_rec, fz_text *text, char *str, int str_len, float x, float y)
+{
+	fz_font *font = font_rec->font->font;
+	int mask = FT_LOAD_NO_SCALE | FT_LOAD_IGNORE_TRANSFORM;
+
+	while (str_len--)
+	{
+		FT_Fixed adv;
+
+		/* FIXME: convert str from utf8 to WinAnsi */
+		int gid = FT_Get_Char_Index(font->ft_face, *str);
+		fz_add_text(ctx, text, gid, *str++, x, y);
+
+		FT_Get_Advance(font->ft_face, gid, mask, &adv);
+		x += ((float)adv) * font_rec->da_rec.font_size / ((FT_Face)font->ft_face)->units_per_EM;
+	}
+}
+
+static fz_text *layout_text(fz_context *ctx, font_info *font_rec, char *str, float x, float y)
+{
+	fz_matrix tm;
+	fz_font *font = font_rec->font->font;
+	fz_text *text;
+
+	fz_scale(&tm, font_rec->da_rec.font_size, font_rec->da_rec.font_size);
+	text = fz_new_text(ctx, font, &tm, 0);
+
+	fz_try(ctx)
+	{
+		add_text(ctx, font_rec, text, str, strlen(str), x, y);
+	}
+	fz_catch(ctx)
+	{
+		fz_free_text(ctx, text);
+		fz_rethrow(ctx);
+	}
+
+	return text;
+}
+
+static fz_text *fit_text(fz_context *ctx, font_info *font_rec, char *str, fz_rect *bounds)
+{
+	float width = bounds->x1 - bounds->x0;
+	float height = bounds->y1 - bounds->y0;
+	fz_matrix tm;
+	fz_text *text = NULL;
+	text_splitter splitter;
+	float ascender;
+
+	/* Initially aim for one-line of text */
+	font_rec->da_rec.font_size = height / font_rec->lineheight;
+
+	text_splitter_init(&splitter, font_rec, str, width, height, 1);
+
+	fz_var(text);
+	fz_try(ctx)
+	{
+		int i;
+		while (!splitter.done)
+		{
+			/* Try a layout pass */
+			int line = 0;
+			float font_size;
+			float x = 0.0;
+			float y = 0.0;
+
+
+			fz_free_text(ctx, text);
+			text = NULL;
+			font_size = font_rec->da_rec.font_size;
+			fz_scale(&tm, font_size, font_size);
+			text = fz_new_text(ctx, font_rec->font->font, &tm, 0);
+
+			text_splitter_start_pass(&splitter);
+
+			/* Layout unscaled text to a scaled-up width, so that
+			* the scaled-down text will fit the unscaled width */
+
+			while (!splitter.done && line < splitter.max_lines)
+			{
+				/* Layout a line */
+				text_splitter_start_line(&splitter);
+
+				while (!splitter.done && text_splitter_layout(ctx, &splitter))
+				{
+					if (splitter.text[splitter.text_start] != ' ')
+					{
+						float dx, dy;
+						char *word = str+splitter.text_start;
+						int wordlen = splitter.text_end-splitter.text_start;
+
+						text_splitter_move(&splitter, -line, &dx, &dy);
+						x += dx;
+						y += dy;
+						add_text(ctx, font_rec, text, word, wordlen, x, y);
+					}
+				}
+
+				line ++;
+			}
+
+			if (!splitter.done)
+				text_splitter_retry(&splitter);
+		}
+
+		/* Post process text with the scale determined by the splitter
+		 * and with the required offst */
+		fz_pre_scale(&text->trm, splitter.scale, splitter.scale);
+		ascender = font_rec->font->ascent * font_rec->da_rec.font_size * splitter.scale / 1000.0f;
+		for (i = 0; i < text->len; i++)
+		{
+			text->items[i].x = text->items[i].x * splitter.scale + bounds->x0;
+			text->items[i].y = text->items[i].y * splitter.scale + bounds->y1 - ascender;
+		}
+	}
+	fz_catch(ctx)
+	{
+		fz_free_text(ctx, text);
+		fz_rethrow(ctx);
+	}
+
+	return text;
+}
+
+static void rect_center(const fz_rect *rect, fz_point *c)
+{
+	c->x = (rect->x0 + rect->x1) / 2.0f;
+	c->y = (rect->y0 + rect->y1) / 2.0f;
+}
+
+static void center_rect_within_rect(const fz_rect *tofit, const fz_rect *within, fz_matrix *mat)
+{
+	float xscale = (within->x1 - within->x0) / (tofit->x1 - tofit->x0);
+	float yscale = (within->y1 - within->y0) / (tofit->y1 - tofit->y0);
+	float scale = fz_min(xscale, yscale);
+	fz_point tofit_center;
+	fz_point within_center;
+
+	rect_center(within, &within_center);
+	rect_center(tofit, &tofit_center);
+
+	/* Translate "tofit" to be centered on the origin
+	 * Scale "tofit" to a size that fits within "within"
+	 * Translate "tofit" to "within's" center
+	 * Do all the above in reverse order so that we can use the fz_pre_xx functions */
+	fz_translate(mat, within_center.x, within_center.y);
+	fz_pre_scale(mat, scale, scale);
+	fz_pre_translate(mat, -tofit_center.x, -tofit_center.y);
+}
+
+static const float outline_thickness = 15.0f;
+
+static void draw_rounded_rect(fz_context *ctx, fz_path *path)
+{
+	fz_moveto(ctx, path, 20.0, 60.0);
+	fz_curveto(ctx, path, 20.0, 30.0, 30.0, 20.0, 60.0, 20.0);
+	fz_lineto(ctx, path, 340.0, 20.0);
+	fz_curveto(ctx, path, 370.0, 20.0, 380.0, 30.0, 380.0, 60.0);
+	fz_lineto(ctx, path, 380.0, 340.0);
+	fz_curveto(ctx, path, 380.0, 370.0, 370.0, 380.0, 340.0, 380.0);
+	fz_lineto(ctx, path, 60.0, 380.0);
+	fz_curveto(ctx, path, 30.0, 380.0, 20.0, 370.0, 20.0, 340.0);
+	fz_closepath(ctx, path);
+}
+
+static void draw_speech_bubble(fz_context *ctx, fz_path *path)
+{
+	fz_moveto(ctx, path, 199.0f, 315.6f);
+	fz_curveto(ctx, path, 35.6f, 315.6f, 27.0f, 160.8f, 130.2f, 131.77f);
+	fz_curveto(ctx, path, 130.2f, 93.07f, 113.0f, 83.4f, 113.0f, 83.4f);
+	fz_curveto(ctx, path, 138.8f, 73.72f, 173.2f, 83.4f, 190.4f, 122.1f);
+	fz_curveto(ctx, path, 391.64f, 122.1f, 362.4f, 315.6f, 199.0f, 315.6f);
+	fz_closepath(ctx, path);
+}
+
+void pdf_update_text_annot_appearance(pdf_document *doc, pdf_annot *annot)
+{
+	static float white[3] = {1.0, 1.0, 1.0};
+	static float yellow[3] = {1.0, 1.0, 0.0};
+	static float black[3] = {0.0, 0.0, 0.0};
+	fz_context *ctx = doc->ctx;
+	const fz_matrix *page_ctm = &annot->page->ctm;
+	fz_display_list *dlist = NULL;
+	fz_device *dev = NULL;
+	fz_colorspace *cs = NULL;
+	fz_path *path = NULL;
+	fz_stroke_state *stroke = NULL;
+
+	fz_var(path);
+	fz_var(stroke);
+	fz_var(dlist);
+	fz_var(dev);
+	fz_var(cs);
+	fz_try(ctx)
+	{
+		fz_rect rect;
+		fz_rect bounds;
+		fz_matrix tm;
+
+		pdf_to_rect(ctx, pdf_dict_gets(annot->obj, "Rect"), &rect);
+		dlist = fz_new_display_list(ctx);
+		dev = fz_new_list_device(ctx, dlist);
+		stroke = fz_new_stroke_state(ctx);
+		stroke->linewidth = outline_thickness;
+		stroke->linejoin = FZ_LINEJOIN_ROUND;
+
+		path = fz_new_path(ctx);
+		draw_rounded_rect(ctx, path);
+		fz_bound_path(ctx, path, NULL, &fz_identity, &bounds);
+		fz_expand_rect(&bounds, outline_thickness);
+		center_rect_within_rect(&bounds, &rect, &tm);
+		fz_concat(&tm, &tm, page_ctm);
+		cs = fz_device_rgb(ctx);
+		fz_fill_path(dev, path, 0, &tm, cs, yellow, 1.0f);
+		fz_stroke_path(dev, path, stroke, &tm, cs, black, 1.0f);
+		fz_free_path(ctx, path);
+		path = NULL;
+
+		path = fz_new_path(ctx);
+		draw_speech_bubble(ctx, path);
+		fz_fill_path(dev, path, 0, &tm, cs, white, 1.0f);
+		fz_stroke_path(dev, path, stroke, &tm, cs, black, 1.0f);
+
+		fz_transform_rect(&rect, page_ctm);
+		pdf_set_annot_appearance(doc, annot, &rect, dlist);
+
+		/* Drop the cached xobject from the annotation structure to
+		 * force a redraw on next pdf_update_page call */
+		pdf_drop_xobject(ctx, annot->ap);
+		annot->ap = NULL;
+	}
+	fz_always(ctx)
+	{
+		fz_free_device(dev);
+		fz_drop_display_list(ctx, dlist);
+		fz_drop_stroke_state(ctx, stroke);
+		fz_free_path(ctx, path);
+		fz_drop_colorspace(ctx, cs);
+	}
+	fz_catch(ctx)
+	{
+		fz_rethrow(ctx);
+	}
+}
+
+void pdf_update_free_text_annot_appearance(pdf_document *doc, pdf_annot *annot)
+{
+	fz_context *ctx = doc->ctx;
+	const fz_matrix *page_ctm = &annot->page->ctm;
+	pdf_obj *obj = annot->obj;
+	pdf_obj *dr = pdf_dict_getp(annot->page->me, "Resources");
+	fz_display_list *dlist = NULL;
+	fz_device *dev = NULL;
+	font_info font_rec;
+	fz_text *text = NULL;
+	fz_colorspace *cs = NULL;
+
+	memset(&font_rec, 0, sizeof(font_rec));
+
+	/* Set some sane defaults in case the parsing of the font_rec fails */
+	font_rec.da_rec.col_size = 1; /* Default to greyscale */
+	font_rec.da_rec.font_size = 12; /* Default to 12 point */
+
+	fz_var(dlist);
+	fz_var(dev);
+	fz_var(text);
+	fz_var(cs);
+	fz_try(ctx)
+	{
+		char *contents = pdf_to_str_buf(pdf_dict_gets(obj, "Contents"));
+		char *da = pdf_to_str_buf(pdf_dict_gets(obj, "DA"));
+		fz_rect rect = annot->rect;
+		fz_point pos;
+
+		get_font_info(doc, dr, da, &font_rec);
+
+		switch (font_rec.da_rec.col_size)
+		{
+		default: cs = fz_device_gray(doc->ctx); break;
+		case 3: cs = fz_device_rgb(doc->ctx); break;
+		case 4: cs = fz_device_cmyk(doc->ctx); break;
+		}
+
+		/* Adjust for the descender */
+		pos.x = rect.x0;
+		pos.y = rect.y0 - font_rec.font->descent * font_rec.da_rec.font_size / 1000.0f;
+
+		text = layout_text(ctx, &font_rec, contents, pos.x, pos.y);
+
+		dlist = fz_new_display_list(ctx);
+		dev = fz_new_list_device(ctx, dlist);
+		fz_fill_text(dev, text, page_ctm, cs, font_rec.da_rec.col, 1.0f);
+
+		fz_transform_rect(&rect, page_ctm);
+		pdf_set_annot_appearance(doc, annot, &rect, dlist);
+	}
+	fz_always(ctx)
+	{
+		fz_free_device(dev);
+		fz_drop_display_list(ctx, dlist);
+		font_info_fin(ctx, &font_rec);
+		fz_free_text(ctx, text);
+		fz_drop_colorspace(ctx, cs);
+	}
+	fz_catch(ctx)
+	{
+		fz_rethrow(ctx);
+	}
+}
+
+static void draw_logo(fz_context *ctx, fz_path *path)
+{
+	fz_moveto(ctx, path, 122.25f, 0.0f);
+	fz_lineto(ctx, path, 122.25f, 14.249f);
+	fz_curveto(ctx, path, 125.98f, 13.842f, 129.73f, 13.518f, 133.5f, 13.277f);
+	fz_lineto(ctx, path, 133.5f, 0.0f);
+	fz_lineto(ctx, path, 122.25f, 0.0f);
+	fz_closepath(ctx, path);
+	fz_moveto(ctx, path, 140.251f, 0.0f);
+	fz_lineto(ctx, path, 140.251f, 12.935f);
+	fz_curveto(ctx, path, 152.534f, 12.477f, 165.03f, 12.899f, 177.75f, 14.249f);
+	fz_lineto(ctx, path, 177.75f, 21.749f);
+	fz_curveto(ctx, path, 165.304f, 20.413f, 152.809f, 19.871f, 140.251f, 20.348f);
+	fz_lineto(ctx, path, 140.251f, 39.0f);
+	fz_lineto(ctx, path, 133.5f, 39.0f);
+	fz_lineto(ctx, path, 133.5f, 20.704f);
+	fz_curveto(ctx, path, 129.756f, 20.956f, 126.006f, 21.302f, 122.25f, 21.749f);
+	fz_lineto(ctx, path, 122.25f, 50.999f);
+	fz_lineto(ctx, path, 177.751f, 50.999f);
+	fz_lineto(ctx, path, 177.751f, 0.0f);
+	fz_lineto(ctx, path, 140.251f, 0.0f);
+	fz_closepath(ctx, path);
+	fz_moveto(ctx, path, 23.482f, 129.419f);
+	fz_curveto(ctx, path, -20.999f, 199.258f, -0.418f, 292.039f, 69.42f, 336.519f);
+	fz_curveto(ctx, path, 139.259f, 381.0f, 232.04f, 360.419f, 276.52f, 290.581f);
+	fz_curveto(ctx, path, 321.001f, 220.742f, 300.42f, 127.961f, 230.582f, 83.481f);
+	fz_curveto(ctx, path, 160.743f, 39.0f, 67.962f, 59.581f, 23.482f, 129.419f);
+	fz_closepath(ctx, path);
+	fz_moveto(ctx, path, 254.751f, 128.492f);
+	fz_curveto(ctx, path, 303.074f, 182.82f, 295.364f, 263.762f, 237.541f, 309.165f);
+	fz_curveto(ctx, path, 179.718f, 354.568f, 93.57f, 347.324f, 45.247f, 292.996f);
+	fz_curveto(ctx, path, -3.076f, 238.668f, 4.634f, 157.726f, 62.457f, 112.323f);
+	fz_curveto(ctx, path, 120.28f, 66.92f, 206.428f, 74.164f, 254.751f, 128.492f);
+	fz_closepath(ctx, path);
+	fz_moveto(ctx, path, 111.0f, 98.999f);
+	fz_curveto(ctx, path, 87.424f, 106.253f, 68.25f, 122.249f, 51.75f, 144.749f);
+	fz_lineto(ctx, path, 103.5f, 297.749f);
+	fz_lineto(ctx, path, 213.75f, 298.499f);
+	fz_curveto(ctx, path, 206.25f, 306.749f, 195.744f, 311.478f, 185.25f, 314.249f);
+	fz_curveto(ctx, path, 164.22f, 319.802f, 141.22f, 319.775f, 120.0f, 314.999f);
+	fz_curveto(ctx, path, 96.658f, 309.745f, 77.25f, 298.499f, 55.5f, 283.499f);
+	fz_curveto(ctx, path, 69.75f, 299.249f, 84.617f, 311.546f, 102.75f, 319.499f);
+	fz_curveto(ctx, path, 117.166f, 325.822f, 133.509f, 327.689f, 149.25f, 327.749f);
+	fz_curveto(ctx, path, 164.21f, 327.806f, 179.924f, 326.532f, 193.5f, 320.249f);
+	fz_curveto(ctx, path, 213.95f, 310.785f, 232.5f, 294.749f, 245.25f, 276.749f);
+	fz_lineto(ctx, path, 227.25f, 276.749f);
+	fz_curveto(ctx, path, 213.963f, 276.749f, 197.25f, 263.786f, 197.25f, 250.499f);
+	fz_lineto(ctx, path, 197.25f, 112.499f);
+	fz_curveto(ctx, path, 213.75f, 114.749f, 228.0f, 127.499f, 241.5f, 140.999f);
+	fz_curveto(ctx, path, 231.75f, 121.499f, 215.175f, 109.723f, 197.25f, 101.249f);
+	fz_curveto(ctx, path, 181.5f, 95.249f, 168.412f, 94.775f, 153.0f, 94.499f);
+	fz_curveto(ctx, path, 139.42f, 94.256f, 120.75f, 95.999f, 111.0f, 98.999f);
+	fz_closepath(ctx, path);
+	fz_moveto(ctx, path, 125.25f, 105.749f);
+	fz_lineto(ctx, path, 125.25f, 202.499f);
+	fz_lineto(ctx, path, 95.25f, 117.749f);
+	fz_curveto(ctx, path, 105.75f, 108.749f, 114.0f, 105.749f, 125.25f, 105.749f);
+	fz_closepath(ctx, path);
+};
+
+static void insert_signature_appearance_layers(pdf_document *doc, pdf_annot *annot)
+{
+	fz_context *ctx = doc->ctx;
+	pdf_obj *ap = pdf_dict_getp(annot->obj, "AP/N");
+	pdf_obj *main_ap = NULL;
+	pdf_obj *frm = NULL;
+	pdf_obj *n0 = NULL;
+	fz_rect bbox;
+	fz_buffer *fzbuf = NULL;
+
+	pdf_to_rect(ctx, pdf_dict_gets(ap, "BBox"), &bbox);
+
+	fz_var(main_ap);
+	fz_var(frm);
+	fz_var(n0);
+	fz_var(fzbuf);
+	fz_try(ctx)
+	{
+		main_ap = pdf_new_xobject(doc, &bbox, &fz_identity);
+		frm = pdf_new_xobject(doc, &bbox, &fz_identity);
+		n0 = pdf_new_xobject(doc, &bbox, &fz_identity);
+
+		pdf_dict_putp(main_ap, "Resources/XObject/FRM", frm);
+		fzbuf = fz_new_buffer(ctx, 8);
+		fz_buffer_printf(ctx, fzbuf, "/FRM Do");
+		pdf_update_stream(doc, pdf_to_num(main_ap), fzbuf);
+		pdf_dict_puts_drop(main_ap, "Length", pdf_new_int(doc, fzbuf->len));
+		fz_drop_buffer(ctx, fzbuf);
+		fzbuf = NULL;
+
+		pdf_dict_putp(frm, "Resources/XObject/n0", n0);
+		pdf_dict_putp(frm, "Resources/XObject/n2", ap);
+		fzbuf = fz_new_buffer(ctx, 8);
+		fz_buffer_printf(ctx, fzbuf, "q 1 0 0 1 0 0 cm /n0 Do Q q 1 0 0 1 0 0 cm /n2 Do Q");
+		pdf_update_stream(doc, pdf_to_num(frm), fzbuf);
+		pdf_dict_puts_drop(frm, "Length", pdf_new_int(doc, fzbuf->len));
+		fz_drop_buffer(ctx, fzbuf);
+		fzbuf = NULL;
+
+		fzbuf = fz_new_buffer(ctx, 8);
+		fz_buffer_printf(ctx, fzbuf, "%% DSBlank");
+		pdf_update_stream(doc, pdf_to_num(n0), fzbuf);
+		pdf_dict_puts_drop(n0, "Length", pdf_new_int(doc, fzbuf->len));
+		fz_drop_buffer(ctx, fzbuf);
+		fzbuf = NULL;
+
+		pdf_dict_putp(annot->obj, "AP/N", main_ap);
+	}
+	fz_always(ctx)
+	{
+		pdf_drop_obj(main_ap);
+		pdf_drop_obj(frm);
+		pdf_drop_obj(n0);
+	}
+	fz_catch(ctx)
+	{
+		fz_drop_buffer(ctx, fzbuf);
+		fz_rethrow(ctx);
+	}
+}
+
+/* MuPDF blue */
+static float logo_color[3] = {(float)0x25/(float)0xFF, (float)0x72/(float)0xFF, (float)0xAC/(float)0xFF};
+
+void pdf_set_signature_appearance(pdf_document *doc, pdf_annot *annot, char *name, char *dn, char *date)
+{
+	fz_context *ctx = doc->ctx;
+	const fz_matrix *page_ctm = &annot->page->ctm;
+	pdf_obj *obj = annot->obj;
+	pdf_obj *dr = pdf_dict_getp(pdf_trailer(doc), "Root/AcroForm/DR");
+	fz_display_list *dlist = NULL;
+	fz_device *dev = NULL;
+	font_info font_rec;
+	fz_text *text = NULL;
+	fz_colorspace *cs = NULL;
+	fz_path *path = NULL;
+	fz_buffer *fzbuf = NULL;
+
+	if (!dr)
+		pdf_dict_putp_drop(pdf_trailer(doc), "Root/AcroForm/DR", pdf_new_dict(doc, 1));
+
+	memset(&font_rec, 0, sizeof(font_rec));
+
+	fz_var(path);
+	fz_var(dlist);
+	fz_var(dev);
+	fz_var(text);
+	fz_var(cs);
+	fz_var(fzbuf);
+	fz_try(ctx)
+	{
+		char *da = pdf_to_str_buf(pdf_dict_gets(obj, "DA"));
+		fz_rect rect = annot->rect;
+		fz_rect logo_bounds;
+		fz_matrix logo_tm;
+		unsigned char *bufstr;
+
+		dlist = fz_new_display_list(ctx);
+		dev = fz_new_list_device(ctx, dlist);
+
+		path = fz_new_path(ctx);
+		draw_logo(ctx, path);
+		fz_bound_path(ctx, path, NULL, &fz_identity, &logo_bounds);
+		center_rect_within_rect(&logo_bounds, &rect, &logo_tm);
+		fz_concat(&logo_tm, &logo_tm, page_ctm);
+		cs = fz_device_rgb(ctx);
+		fz_fill_path(dev, path, 0, &logo_tm, cs, logo_color, 1.0f);
+		fz_drop_colorspace(ctx, cs);
+		cs = NULL;
+
+		get_font_info(doc, dr, da, &font_rec);
+
+		switch (font_rec.da_rec.col_size)
+		{
+		case 1: cs = fz_device_gray(ctx); break;
+		case 3: cs = fz_device_rgb(ctx); break;
+		case 4: cs = fz_device_cmyk(ctx); break;
+		}
+
+		/* Display the name in the left-hand half of the form field */
+		rect.x1 = (rect.x0 + rect.x1)/2.0f;
+		text = fit_text(ctx, &font_rec, name, &rect);
+		fz_fill_text(dev, text, page_ctm, cs, font_rec.da_rec.col, 1.0f);
+		fz_free_text(ctx, text);
+		text = NULL;
+
+		/* Display the distinguished name in the right-hand half */
+		fzbuf = fz_new_buffer(ctx, 256);
+		fz_buffer_printf(ctx, fzbuf, "Digitally signed by %s", name);
+		fz_buffer_printf(ctx, fzbuf, "\nDN: %s", dn);
+		if (date)
+			fz_buffer_printf(ctx, fzbuf, "\nDate: %s", date);
+		(void)fz_buffer_storage(ctx, fzbuf, &bufstr);
+		rect = annot->rect;
+		rect.x0 = (rect.x0 + rect.x1)/2.0f;
+		text = fit_text(ctx, &font_rec, (char *)bufstr, &rect);
+		fz_fill_text(dev, text, page_ctm, cs, font_rec.da_rec.col, 1.0f);
+
+		rect = annot->rect;
+		fz_transform_rect(&rect, page_ctm);
+		pdf_set_annot_appearance(doc, annot, &rect, dlist);
+
+		/* Drop the cached xobject from the annotation structure to
+		 * force a redraw on next pdf_update_page call */
+		pdf_drop_xobject(ctx, annot->ap);
+		annot->ap = NULL;
+
+		insert_signature_appearance_layers(doc, annot);
+	}
+	fz_always(ctx)
+	{
+		fz_free_device(dev);
+		fz_drop_display_list(ctx, dlist);
+		font_info_fin(ctx, &font_rec);
+		fz_free_path(ctx, path);
+		fz_free_text(ctx, text);
+		fz_drop_colorspace(ctx, cs);
+		fz_drop_buffer(ctx, fzbuf);
+	}
+	fz_catch(ctx)
+	{
+		fz_rethrow(ctx);
+	}
+}
+
+void pdf_update_appearance(pdf_document *doc, pdf_annot *annot)
+{
+	pdf_obj *obj = annot->obj;
+	if (!pdf_dict_gets(obj, "AP") || pdf_obj_is_dirty(obj))
+	{
+		fz_annot_type type = pdf_annot_obj_type(obj);
+		switch (type)
+		{
+		case FZ_ANNOT_WIDGET:
+			switch (pdf_field_type(doc, obj))
+			{
+			case PDF_WIDGET_TYPE_TEXT:
+				{
+					#if 0
+					pdf_obj *formatting = pdf_dict_getp(obj, "AA/F");
+					if (formatting && doc->js)
+					{
+						/* Apply formatting */
+						pdf_js_event e;
+						fz_context *ctx = doc->ctx;
+
+						e.target = obj;
+						e.value = pdf_field_value(doc, obj);
+						fz_try(ctx)
+						{
+							pdf_js_setup_event(doc->js, &e);
+						}
+						fz_always(ctx)
+						{
+							fz_free(ctx, e.value);
+						}
+						fz_catch(ctx)
+						{
+							fz_rethrow(ctx);
+						}
+						execute_action(doc, obj, formatting);
+						/* Update appearance from JS event.value */
+						pdf_update_text_appearance(doc, obj, pdf_js_get_event(doc->js)->value);
+					}
+					else
+					#endif
+					{
+						/* Update appearance from field value */
+						pdf_update_text_appearance(doc, obj, NULL);
+					}
+				}
+				break;
+			case PDF_WIDGET_TYPE_PUSHBUTTON:
+				pdf_update_pushbutton_appearance(doc, obj);
+				break;
+			case PDF_WIDGET_TYPE_LISTBOX:
+			case PDF_WIDGET_TYPE_COMBOBOX:
+				/* Treating listbox and combobox identically for now,
+				 * and the behaviour is most appropriate for a combobox */
+				pdf_update_combobox_appearance(doc, obj);
+				break;
+			}
+			break;
+		case FZ_ANNOT_TEXT:
+			pdf_update_text_annot_appearance(doc, annot);
+			break;
+		case FZ_ANNOT_FREETEXT:
+			pdf_update_free_text_annot_appearance(doc, annot);
+			break;
+		case FZ_ANNOT_STRIKEOUT:
+		case FZ_ANNOT_UNDERLINE:
+		case FZ_ANNOT_HIGHLIGHT:
+			pdf_update_text_markup_appearance(doc, annot, type);
+			break;
+		case FZ_ANNOT_INK:
+			pdf_update_ink_appearance(doc, annot);
+			break;
+		default:
+			break;
+		}
+
+		pdf_clean_obj(obj);
+	}
+}
