@@ -8,22 +8,12 @@
 	resulting executables.
 */
 
-static void pdf_run_page_contents_shim(fz_document *doc, fz_page *page, fz_device *dev, const fz_matrix *transform, fz_cookie *cookie)
-{
-	pdf_run_page_contents((pdf_document*)doc, (pdf_page*)page, dev, transform, cookie);
-}
-
-static void pdf_run_annot_shim(fz_document *doc, fz_page *page, fz_annot *annot, fz_device *dev, const fz_matrix *transform, fz_cookie *cookie)
-{
-	pdf_run_annot((pdf_document*)doc, (pdf_page*)page, (pdf_annot *)annot, dev, transform, cookie);
-}
-
 pdf_document *
 pdf_open_document_with_stream(fz_context *ctx, fz_stream *file)
 {
 	pdf_document *doc = pdf_open_document_no_run_with_stream(ctx, file);
-	doc->super.run_page_contents = pdf_run_page_contents_shim;
-	doc->super.run_annot = pdf_run_annot_shim;
+	doc->super.run_page_contents = (fz_document_run_page_contents_fn *)pdf_run_page_contents;
+	doc->super.run_annot = (fz_document_run_annot_fn *)pdf_run_annot;
 	doc->update_appearance = pdf_update_appearance;
 	return doc;
 }
@@ -32,8 +22,15 @@ pdf_document *
 pdf_open_document(fz_context *ctx, const char *filename)
 {
 	pdf_document *doc = pdf_open_document_no_run(ctx, filename);
-	doc->super.run_page_contents = pdf_run_page_contents_shim;
-	doc->super.run_annot = pdf_run_annot_shim;
+	doc->super.run_page_contents = (fz_document_run_page_contents_fn *)pdf_run_page_contents;
+	doc->super.run_annot = (fz_document_run_annot_fn *)pdf_run_annot;
 	doc->update_appearance = pdf_update_appearance;
 	return doc;
 }
+
+fz_document_handler pdf_document_handler =
+{
+	(fz_document_recognize_fn *)&pdf_recognize,
+	(fz_document_open_fn *)&pdf_open_document,
+	(fz_document_open_with_stream_fn *)&pdf_open_document_with_stream
+};
